use crate::Res;

#[derive(Debug, Clone, serde::Deserialize)]
pub struct OpenIDConfig {
    pub authorization_endpoint: String,
    pub token_endpoint: String,
    pub userinfo_endpoint: String,
}

impl OpenIDConfig {
    /// Load OpenID configuration from a given URL
    pub async fn load_from(url: &str) -> Res<Self> {
        Ok(reqwest::get(url).await?.json().await?)
    }

    /// Get the authorization URL where a user should be redirect
    pub fn authorization_url(&self, client_id: &str, state: &str, redirect_uri: &str) -> String {
        let client_id = urlencoding::encode(client_id);
        let state = urlencoding::encode(state);
        let redirect_uri = urlencoding::encode(redirect_uri);

        format!("{}?response_type=code&scope=openid%20profile%20email&client_id={client_id}&state={state}&redirect_uri={redirect_uri}", self.authorization_endpoint)
    }
}
